@@ -172,14 +172,18 @@ sub _table {
   my $sep = $self->{name_sep} // '';
   for my $join (@join) {
     puke 'join must be in the form [$table, $fk => $pk]' if @$join < 3;
-    my $type = @$join > 3 ? shift @$join : '';
-    my ($name, $fk, $pk) = @$join;
+    my $type = @$join % 2 == 0 ? shift @$join : '';
+    my ($name, $fk, $pk, @morekeys) = @$join;
     $table
       .= $self->_sqlcase($type =~ /^-(.+)$/ ? " $1 join " : ' join ')
       . $self->_quote($name)
-      . $self->_sqlcase(' on ') . '('
-      . $self->_quote(index($fk, $sep) > 0 ? $fk : "$name.$fk") . ' = '
-      . $self->_quote(index($pk, $sep) > 0 ? $pk : "$table[0].$pk") . ')';
+      . $self->_sqlcase(' on ') . '(';
+    do {
+      $table
+        .= $self->_quote(index($fk, $sep) > 0 ? $fk : "$name.$fk") . ' = '
+        . $self->_quote(index($pk, $sep) > 0 ? $pk : "$table[0].$pk")
+        . (@morekeys ? $self->_sqlcase(' and ') : ')');
+    } while ($fk, $pk, @morekeys) = @morekeys;
   }
 
   return $table;
@@ -290,6 +294,9 @@ for.
   # "select * from foo left join bar on (bar.foo_id = foo.id)"
   $abstract->select(['foo', [-left => 'bar', foo_id => 'id']]);
 
+  # "select * from a left join b on (b.a_id = a.id and b.a_id2 = a.id2)"
+  $abstract->select(['a', [-left => 'b', a_id => 'id', a_id2 => 'id2']]);
+
 =head2 ORDER BY
 
 Alternatively to the C<$order> argument accepted by L<SQL::Abstract> you can now
